----------------------------------------------------------------------------
-- @Author: ViperGTS96-----------------------------------------------------
----------------------------------------------------------------------------
--------------------"The simplest design is the best design." --------------
----------------------------------------------------------------------------
----------------------------------------------------------------------------

bigfootAnimations = {};

function bigfootAnimations.prerequisitesPresent(specializations)
	return SpecializationUtil.hasSpecialization(Washable, specializations)
		and SpecializationUtil.hasSpecialization(Motorized, specializations)
		and SpecializationUtil.hasSpecialization(AnimatedVehicle, specializations);
end;

if GS_IS_CONSOLE_VERSION then return; end;

function bigfootAnimations.registerEventListeners(vehicleType)
	SpecializationUtil.registerEventListener(vehicleType, "onLoad", bigfootAnimations);
	SpecializationUtil.registerEventListener(vehicleType, "onPostLoad", bigfootAnimations);
	SpecializationUtil.registerEventListener(vehicleType, "onUpdate", bigfootAnimations);
end;

function bigfootAnimations:onLoad(savegame)
	self.spec_bigfootAnim = {};
	if hasXMLProperty(self.xmlFile, "vehicle.animations") then
		self.spec_bigfootAnim.lightKnob = "lightKnobPull";
		self.spec_bigfootAnim.wiperKnob = "wiperKnobPull";
		self.spec_bigfootAnim.shifter = I3DUtil.indexToObject(self.components, getXMLString(self.xmlFile, "vehicle.motorized.shifter#node"), self.i3dMappings);
		self.spec_bigfootAnim.lightNode = I3DUtil.indexToObject(self.components, getXMLString(self.xmlFile, string.format("vehicle.lights.realLights.low.light(%d)#node",4)), self.i3dMappings);
		if self:getAnimationExists(self.spec_bigfootAnim.lightKnob) then
			self.updateLightSwitchKnob = bigfootAnimations.updateLightSwitchKnob;
		else
			self.spec_bigfootAnim.lightKnob = nil;
			print("bigfootAnimations: Error locating lightKnobPull animation");
		end;
		if self:getAnimationExists(self.spec_bigfootAnim.wiperKnob) then
			self.updateWiperKnob = bigfootAnimations.updateWiperKnob;
		else
			self.spec_bigfootAnim.wiperKnob = nil;
			print("bigfootAnimations: Error locating wiperKnobPull animation");
		end;
		if self.spec_bigfootAnim.lightNode == nil then
			print("bigfootAnimations: Error locating dashlightNode");
		end;
		if self.spec_bigfootAnim.shifter ~= nil then
			self.updateShifterPark = bigfootAnimations.updateShifterPark;
			self.spec_bigfootAnim.shifterInPark = false;
		else
			print("bigfootAnimations: Error locating shifterNode");
		end;
	end;
end;

function bigfootAnimations:onPostLoad(savegame)
	local hasIC = self.spec_iC ~= nil;
	if hasIC then 
		hasIC = self.spec_iC.wiperSwitchAnimation ~= nil and self.spec_iC.wiperSwitchAnimation;
		if hasIC then
			self.spec_bigfootAnim.wiperKnob = nil;
		end;
	end;
end;

function bigfootAnimations:updateShifterPark()
	if self:getIsMotorStarted() then
		if self.spec_bigfootAnim.shifterInPark then
			if not self:getIsAnimationPlaying("shiftGears") then
				setTranslation(self.spec_bigfootAnim.shifter, 0.018,2.0975101,0.50967598);
				setRotation(self.spec_bigfootAnim.shifter, 0,0,0);
				self.spec_bigfootAnim.shifterInParkDelay = nil;
				self.spec_bigfootAnim.shifterInPark = false;
			end;
		end;
	else
		if not self.spec_bigfootAnim.shifterInPark then
			if self.spec_bigfootAnim.shifterInParkDelay == nil then
				self.spec_bigfootAnim.shifterInParkDelay = (g_time/1000) + 2;
			elseif (g_time/1000) >= self.spec_bigfootAnim.shifterInParkDelay then
				if not self:getIsAnimationPlaying("shiftGears") then
					setTranslation(self.spec_bigfootAnim.shifter, 0.018,2.09530401,0.54958165);
					setRotation(self.spec_bigfootAnim.shifter, math.rad(12),0,0);
					self.spec_bigfootAnim.shifterInPark = true;
				end;
			end;
		end;
	end;
end;

function bigfootAnimations:updateWiperKnob()
	if self:getIsEntered() then
		local wipersOn = false;
		if Wipers.forcedState > 0 then
			wipersOn = true;
		else
			local rainScale = g_currentMission.environment.weather:getRainFallScale()
			local timeSinceLastRain = g_currentMission.environment.weather:getTimeSinceLastRain()
			if not (rainScale <= 0.1 and timeSinceLastRain > 20) then
				wipersOn = true;
			end;
		end;
		if wipersOn then
			local animName = self.spec_bigfootAnim.wiperKnob;
			if not self:getIsAnimationPlaying(animName) then
				if self:getAnimationTime(animName) <= 0 then
					self:playAnimation(animName, 1);
				end;
			end;
		else
			local animName = self.spec_bigfootAnim.wiperKnob;
			if not self:getIsAnimationPlaying(animName) then
				if self:getAnimationTime(animName) >= 0.1 then
					self:playAnimation(animName, -1);
				end;
			end;
		end;
	else
		local animName = self.spec_bigfootAnim.wiperKnob;
		if not self:getIsAnimationPlaying(animName) then
			if self:getAnimationTime(animName) >= 0.1 then
				self:playAnimation(animName, -1);
			end;
		end;
	end;
end;

function bigfootAnimations:updateLightSwitchKnob()
	if getVisibility(self.spec_bigfootAnim.lightNode) then
		local animName = self.spec_bigfootAnim.lightKnob;
		if not self:getIsAnimationPlaying(animName) then
			if self:getAnimationTime(animName) <= 0 then
				self:playAnimation(animName, 1);
			end;
		end;
	else
		local animName = self.spec_bigfootAnim.lightKnob;
		if not self:getIsAnimationPlaying(animName) then
			if self:getAnimationTime(animName) >= 0.1 then
				self:playAnimation(animName, -1);
			end;
		end;
	end;
end;

function bigfootAnimations:onUpdate(dt)
	if self.spec_bigfootAnim.lightKnob ~= nil then
		self:updateLightSwitchKnob();
	end;
	if self.spec_bigfootAnim.wiperKnob ~= nil then
		self:updateWiperKnob();
	end;
	if self.spec_bigfootAnim.shifter ~= nil then
		self:updateShifterPark();
	end;
end;