----------------------------------------------------------------------------
-- @Author: ViperGTS96-----------------------------------------------------
----------------------------------------------------------------------------
--------------------"The simplest design is the best design." --------------
----------------------------------------------------------------------------
----------------------------------------------------------------------------

bigfootVibrations = {};

function bigfootVibrations.prerequisitesPresent(specializations)
	return SpecializationUtil.hasSpecialization(Washable, specializations)
		and SpecializationUtil.hasSpecialization(Motorized, specializations)
		and SpecializationUtil.hasSpecialization(AnimatedVehicle, specializations);
end;

if GS_IS_CONSOLE_VERSION then return; end;
bigfootVibrations.maxRotation = 0.2; --degrees in either direction.
bigfootVibrations.maxLoadRotation = 0.6; --degrees in either direction.
bigfootVibrations.maxCamShake = 0.1; --degrees in either direction.
bigfootVibrations.enginePivotSpeed = 0.1;--0.2
bigfootVibrations.camPivotSpeed = 0.15;--0.2

function bigfootVibrations.registerEventListeners(vehicleType)
	SpecializationUtil.registerEventListener(vehicleType, "onLoad", bigfootVibrations);
	SpecializationUtil.registerEventListener(vehicleType, "onUpdate", bigfootVibrations);
end;

function bigfootVibrations:onLoad(savegame)
	self.enginePivotNode = I3DUtil.indexToObject(self.components, getXMLString(self.xmlFile, "vehicle.motorized#enginePivotNode"), self.i3dMappings);
	self.indoorCamPivotNode = I3DUtil.indexToObject(self.components, getXMLString(self.xmlFile, string.format("vehicle.enterable.cameras.camera(%d)#pivotNode",1)), self.i3dMappings);
	self.characterPivotNode = I3DUtil.indexToObject(self.components, getXMLString(self.xmlFile, "vehicle.enterable.characterNode#node"), self.i3dMappings);
	if self.enginePivotNode == nil then print("Cannot locate Bigfoot's engine pivot node!"); end;
	if self.indoorCamPivotNode == nil then print("Cannot locate Bigfoot's indoor camera pivot node!"); end;
	self.enginePivotRight = false;
	self.indoorCamPivotRight = false;
end;

function bigfootVibrations:shakeCamera(bigfoot, dt)
	local _,_,rot = getRotation(bigfoot.indoorCamPivotNode);
	rot = math.deg(rot);
	if bigfoot.indoorCamPivotRight then
		if rot < bigfootVibrations.maxCamShake-0.0001 then
			rot = math.min(rot+(bigfootVibrations.camPivotSpeed+dt),bigfootVibrations.maxCamShake);
			setRotation(bigfoot.indoorCamPivotNode, 0,0,math.rad(rot));
			setRotation(bigfoot.characterPivotNode, 0,0,math.rad(rot));
		else
			setRotation(bigfoot.indoorCamPivotNode, 0,0,math.rad(bigfootVibrations.maxCamShake));
			setRotation(bigfoot.characterPivotNode, 0,0,math.rad(bigfootVibrations.maxCamShake));
			bigfoot.indoorCamPivotRight = false;
		end;
	else
		if rot > -(bigfootVibrations.maxCamShake-0.0001) then
			rot = math.max(rot-(bigfootVibrations.camPivotSpeed+dt),-(bigfootVibrations.maxCamShake));
			setRotation(bigfoot.indoorCamPivotNode, 0,0,math.rad(rot));
			setRotation(bigfoot.characterPivotNode, 0,0,math.rad(rot));
		else
			setRotation(bigfoot.indoorCamPivotNode, 0,0,math.rad(-bigfootVibrations.maxCamShake));
			setRotation(bigfoot.characterPivotNode, 0,0,math.rad(-bigfootVibrations.maxCamShake));
			bigfoot.indoorCamPivotRight = true;
		end;
	end;
end;

function bigfootVibrations:shakeEngine(bigfoot, dt)
	local _,_,rot = getRotation(bigfoot.enginePivotNode);
	rot = math.deg(rot);
	if bigfoot.enginePivotRight then
		if rot < (bigfootVibrations.maxRotation-0.0001) then
			rot = math.min(rot+(bigfootVibrations.enginePivotSpeed+dt),bigfootVibrations.maxRotation);
			setRotation(bigfoot.enginePivotNode, 0,0,math.rad(rot));
		else
			setRotation(bigfoot.enginePivotNode, 0,0,math.rad(bigfootVibrations.maxRotation));
			bigfoot.enginePivotRight = false;
		end;
	else
		if rot > -(bigfootVibrations.maxRotation-0.0001) then
			rot = math.max(rot-(bigfootVibrations.enginePivotSpeed+dt),-(bigfootVibrations.maxRotation));
			setRotation(bigfoot.enginePivotNode, 0,0,math.rad(rot));
		else
			setRotation(bigfoot.enginePivotNode, 0,0,math.rad(-bigfootVibrations.maxRotation));
			bigfoot.enginePivotRight = true;
		end;
	end;
end;

function bigfootVibrations:applyEngineLoad(bigfoot, underLoad, dt)
	local _,_,rot = getRotation(bigfoot.enginePivotNode);
	rot = math.deg(rot);
	if underLoad then
		if rot < bigfootVibrations.maxRotation-0.0001 then
			rot = math.min(rot+(bigfootVibrations.enginePivotSpeed+dt),bigfootVibrations.maxLoadRotation);
			setRotation(bigfoot.enginePivotNode, 0,0,math.rad(rot));
		else
			setRotation(bigfoot.enginePivotNode, 0,0,math.rad(bigfootVibrations.maxLoadRotation));
			bigfoot.enginePivotRight = false;
		end;
	else
		if rot > 0 then
			rot = math.max(rot-(bigfootVibrations.enginePivotSpeed+dt),0);
			setRotation(bigfoot.enginePivotNode, 0,0,math.rad(rot));
		else
			setRotation(bigfoot.enginePivotNode, 0,0,0);
			bigfoot.enginePivotRight = true;
		end;
	end;
end;

function bigfootVibrations:resetRotations(bigfoot, resetEnginePosition, dt, hardReset)
	if hardReset then
		setRotation(bigfoot.indoorCamPivotNode, 0,0,0);
		setRotation(bigfoot.enginePivotNode, 0,0,0);
		setRotation(bigfoot.characterPivotNode, 0,0,0);
	else
		local _,_,rot = getRotation(bigfoot.indoorCamPivotNode);
		rot = math.deg(rot);
		if rot ~= 0 then
			if rot < 0 then
				rot = math.min(rot+(bigfootVibrations.camPivotSpeed+dt),0);
				setRotation(bigfoot.indoorCamPivotNode, 0,0,math.rad(rot));
				setRotation(bigfoot.characterPivotNode, 0,0,math.rad(rot));
			else
				rot = math.max(rot-(bigfootVibrations.camPivotSpeed+dt),0);
				setRotation(bigfoot.indoorCamPivotNode, 0,0,math.rad(rot));
				setRotation(bigfoot.characterPivotNode, 0,0,math.rad(rot));
			end;
		end;
		if resetEnginePosition then
			_,_,rot = getRotation(bigfoot.enginePivotNode);
			rot = math.deg(rot);
			if rot ~= 0 then
				if rot < 0 then
					rot = math.min(rot+(bigfootVibrations.enginePivotSpeed+dt),0);
					setRotation(bigfoot.enginePivotNode, 0,0,math.rad(rot));
				else
					rot = math.max(rot-(bigfootVibrations.enginePivotSpeed+dt),0);
					setRotation(bigfoot.enginePivotNode, 0,0,math.rad(rot));
				end;
			end;
		end;
	end;
	bigfoot.enginePivotRight = false;
	bigfoot.indoorCamPivotRight = false;
end;

function bigfootVibrations:onUpdate(dt)
	dt = math.max((dt-30)*0.01,0);
	if self:getIsMotorStarted() then
		local engine = self:getMotor();
		if (engine.lastMotorRpm < 1400 or self:getLastSpeed(true)<1) and self:getAccelerationAxis()==0 then
			bigfootVibrations:shakeEngine(self, dt);
			bigfootVibrations:shakeCamera(self, dt);
		else
			local engineLoaded = (self.spec_motorized:getMotorLoadPercentage()>0.001) or (self:getAccelerationAxis()~=0);
			bigfootVibrations:applyEngineLoad(self, engineLoaded, dt);
			bigfootVibrations:resetRotations(self, false, dt, false);
		end;
	else
		bigfootVibrations:resetRotations(self, true, dt, true);
	end;
end;