----------------------------------------------------------------------------
-- @Author: ViperGTS96-----------------------------------------------------
----------------------------------------------------------------------------
--------------------"The simplest design is the best design." --------------
----------------------------------------------------------------------------
----------------------------------------------------------------------------

bigfootWetTires = {};

function bigfootWetTires.prerequisitesPresent(specializations)
	return SpecializationUtil.hasSpecialization(Washable, specializations)
		and SpecializationUtil.hasSpecialization(Motorized, specializations)
		and SpecializationUtil.hasSpecialization(AnimatedVehicle, specializations);
end;

if GS_IS_CONSOLE_VERSION then return; end;

function bigfootWetTires.registerEventListeners(vehicleType)
	SpecializationUtil.registerEventListener(vehicleType, "onLoad", bigfootWetTires);
	SpecializationUtil.registerEventListener(vehicleType, "onPostLoad", bigfootWetTires);
	SpecializationUtil.registerEventListener(vehicleType, "onUpdate", bigfootWetTires);
end;

function bigfootWetTires:onLoad()
	self.spec_bigfootWetTires = {};
	self.spec_bigfootWetTires.wetTireY = g_currentMission.waterY;
	if g_currentMission.waterBodyFX ~= nil then
		self.spec_bigfootWetTires.wetTireY = g_currentMission.waterBodyFX.waterY;
	end;
	local shine,_,_,_ = getShaderParameter(self.spec_wheels.wheels[1].wheelTire, "SM");
	self.spec_bigfootWetTires.tireShineMin = shine;
end;

function bigfootWetTires:onPostLoad()
	for _,wheel in pairs(self.spec_wheels.wheels) do
		if wheel.mudFXMainNode == nil and wheel.waterBodyFXMainNode == nil then
			wheel.wetTireDetectNode = createTransformGroup("wetTireDetectorNode");
			link(wheel.node, wheel.wetTireDetectNode);
			setWorldTranslation(wheel.wetTireDetectNode, unpack({localToWorld(wheel.driveNode,0,-wheel.radius,0)}));
			local rDX1,rDY1,rDZ1 = localToWorld(self.components[1].node, 0,0,1);
			local rDX2,rDY2,rDZ2 = localToWorld(self.components[1].node, 0,1,0);
			setDirection(wheel.wetTireDetectNode,rDX1,rDY1,rDZ1,rDX2,rDY2,rDZ2);
		elseif wheel.mudFXMainNode ~= nil then
			wheel.wetTireDetectNode = wheel.mudFXMainNode;
		elseif wheel.waterBodyFXMainNode ~= nil then
			wheel.wetTireDetectNode = wheel.waterBodyFXMainNode;
		end;
	end;
	bigfootWetVehicleInit:onPostLoad(self);
end;

function bigfootWetTires:onUpdate(dt)
	if self.owner ~= nil then
		dt = dt*0.01;
		local tspeed = self:getLastSpeed(true);
		local isRaining = false;
		if g_currentMission.wetVehicles ~= nil and g_currentMission.wetVehicles.isRaining then isRaining = true; end;
		if not isRaining then
			for _,wheel in pairs(self.spec_wheels.wheels) do
				local x,y,z = getWorldTranslation(wheel.wetTireDetectNode);
				local y2 = y+(wheel.radius*2);
				local shine,metalness,_,_ = getShaderParameter(wheel.wheelTire, "SM");
				if y2 <= self.spec_bigfootWetTires.wetTireY then
					if shine < 1 then
						if wheel.hasGroundContact ~= nil and wheel.hasGroundContact then
							setShaderParameter(wheel.wheelTire, "SM", shine+(0.1*dt), metalness, 0, 0, false);
						end;
					end;
				elseif y <= self.spec_bigfootWetTires.wetTireY and tspeed > 1 then
					if shine < 1 then
						if wheel.hasGroundContact ~= nil and wheel.hasGroundContact then
							tspeed = MathUtil.clamp(tspeed/2,1,10);
							setShaderParameter(wheel.wheelTire, "SM", shine+((tspeed*0.01)*dt), metalness, 0, 0, false);--0.01*dt
						end;
					end;
				else
					if shine > self.spec_bigfootWetTires.tireShineMin then
						if wheel.hasGroundContact ~= nil and not wheel.hasGroundContact then
							tspeed = 1;
						else
							tspeed = MathUtil.clamp(tspeed,2,20);
						end;
						setShaderParameter(wheel.wheelTire, "SM", shine-((tspeed*0.001)*dt), metalness, 0, 0, false);
					elseif shine ~= self.spec_bigfootWetTires.tireShineMin then
						setShaderParameter(wheel.wheelTire, "SM", self.spec_bigfootWetTires.tireShineMin, metalness, 0, 0, false);
					end;
				end;
			end;
		end;
		bigfootCrashMonitor:onUpdate(self, tspeed);
	end;
end;

bigfootCrashMonitor = {};

function bigfootCrashMonitor:onUpdate(self, speed)
	local speed = self:getLastSpeed(true);
	if speed > 1000.0 then
		print("Vehicle encounterd unkown catastrophic collision error");
		if not g_currentMission.controlPlayer and g_currentMission.controlledVehicle ~= nil and self == g_currentMission.controlledVehicle then
            g_currentMission:onLeaveVehicle(g_currentMission.playerStartX+math.random(1.0,3.0), g_currentMission.playerStartY, g_currentMission.playerStartZ+math.random(1.0,3.0));
        end
        g_client:getServerConnection():sendEvent(ResetVehicleEvent:new(self));
	end;
end;

source(g_currentModDirectory.."scripts/bigfootWetVehicleInit.lua");