----------------------------------------------------------------------------
-- @Author: ViperGTS96-----------------------------------------------------
----------------------------------------------------------------------------
--------------------"The simplest design is the best design." --------------
----------------------------------------------------------------------------
----------------------------------------------------------------------------

bigfootWorkMode = {};

function bigfootWorkMode.prerequisitesPresent(specializations)
	return SpecializationUtil.hasSpecialization(Washable, specializations)
		and SpecializationUtil.hasSpecialization(Motorized, specializations)
		and SpecializationUtil.hasSpecialization(AnimatedVehicle, specializations);
end;

if GS_IS_CONSOLE_VERSION then return; end;
bigfootWorkMode.addedMass = 7500/1000; --all mass is to be added to axles due to cabin suspension on body.

function bigfootWorkMode.registerEventListeners(vehicleType)
	SpecializationUtil.registerEventListener(vehicleType, "onLoad", bigfootWorkMode);
	SpecializationUtil.registerEventListener(vehicleType, "onUpdate", bigfootWorkMode);
end;

function bigfootWorkMode:onLoad(savegame)
	self.workMode = false;
	self.defaultAccelLimit = self:getMotor():getAccelerationLimit(); --register original acceleration limit
	self.defaultAxleMass = self.components[2].mass; --register original axle mass
	self.defaultTireFriction = self.spec_wheels.wheels[1].frictionScale;--register original wheel/tire friction scale
	--self.defaultWheelSpring = self.spec_wheels.wheels[1].spring;--register original wheel/tire spring rate
	local _,_,z = StringUtil.getVectorFromString(getXMLString(self.xmlFile, string.format("vehicle.base.components.joint(%d)#rotLimit",1)));
	self.defaultAxleRotLimit = math.rad(z);--register rear axle rotation(z) limit
end;

function bigfootWorkMode:onUpdate(dt)
	local attachedImplements;
	if self.getAttachedImplements ~= nil then
		attachedImplements = self:getAttachedImplements();
	end;
	if attachedImplements ~= nil and #attachedImplements > 0 then -- if attached to any tools/trailers 'work-mode' is enabled.
		if not self.workMode then --set work mode parameters
			setMass(self.components[2].node, self.defaultAxleMass+bigfootWorkMode.addedMass);--increase front axle mass
			setMass(self.components[3].node, self.defaultAxleMass+bigfootWorkMode.addedMass);--increase rear axle mass
			self:getMotor():setAccelerationLimit(self.defaultAccelLimit/5);--low gear
			for _, wheel in pairs(self.spec_wheels.wheels) do
				wheel.frictionScale = wheel.frictionScale*2;--increase wheel friction
				--wheel.spring = wheel.spring*2;--increase wheel spring
				Wheels:updateWheelTireFriction(wheel);--update wheel friction
			end;
			self:setComponentJointRotLimit(self.componentJoints[2], 3, 0, 0); --lock Rear Axle
			self.workMode = true;
		end;
	else
		if self.workMode then --disable work mode; set default parameters
			setMass(self.components[2].node, self.defaultAxleMass);--restore default front axle mass
			setMass(self.components[3].node, self.defaultAxleMass);--restore default rear axle mass
			self:getMotor():setAccelerationLimit(self.defaultAccelLimit);--high gear
			for _, wheel in pairs(self.spec_wheels.wheels) do
				wheel.frictionScale = self.defaultTireFriction;--restore wheel friction
				--wheel.spring = self.defaultWheelSpring;--restore wheel spring
				Wheels:updateWheelTireFriction(wheel);--update wheel friction
			end;
			self:setComponentJointRotLimit(self.componentJoints[2], 3, -(self.defaultAxleRotLimit), self.defaultAxleRotLimit);--unlock Rear Axle
			self.workMode = false;
		end;
	end;
end;