----------------------------------------------------------------------------
-- @Author: ViperGTS96------------------------------------------------------
----------------------------------------------------------------------------
--------------------"The simplest design is the best design." --------------
----------------------------------------------------------------------------
----------------------------------------------------------------------------

interactiveControls = {}
interactiveControls.debugging = false;

function interactiveControls.prerequisitesPresent(specializations)
	return SpecializationUtil.hasSpecialization(Motorized, specializations)
		and SpecializationUtil.hasSpecialization(AnimatedVehicle, specializations);
end;

function interactiveControls.registerFunctions(vehicleType)
	SpecializationUtil.registerFunction(vehicleType, "loadIControls", interactiveControls.loadIControls);
	SpecializationUtil.registerFunction(vehicleType, "setLightTypeAndMsg", interactiveControls.setLightTypeAndMsg);
	SpecializationUtil.registerFunction(vehicleType, "useIControl", interactiveControls.useIControl);
	SpecializationUtil.registerFunction(vehicleType, "useIControlEvent", interactiveControls.useIControlEvent);
	SpecializationUtil.registerFunction(vehicleType, "updateIControls", interactiveControls.updateIControls);
	SpecializationUtil.registerFunction(vehicleType, "updateICtrlMsgs", interactiveControls.updateICtrlMsgs);
	SpecializationUtil.registerFunction(vehicleType, "updateICtrlSounds", interactiveControls.updateICtrlSounds);
	SpecializationUtil.registerFunction(vehicleType, "updateIndoorSounds", interactiveControls.updateIndoorSounds);
	SpecializationUtil.registerFunction(vehicleType, "updateICWipers", interactiveControls.updateICWipers);
	SpecializationUtil.registerFunction(vehicleType, "getIsIndoorCamera", interactiveControls.getIsIndoorCamera);
	SpecializationUtil.registerFunction(vehicleType, "loadICSound", interactiveControls.loadICSound);
	SpecializationUtil.registerFunction(vehicleType, "playICSound", interactiveControls.playICSound);
	SpecializationUtil.registerFunction(vehicleType, "checkICCamera", interactiveControls.checkICCamera);
	SpecializationUtil.registerFunction(vehicleType, "killIC", interactiveControls.killIC);
end;

function interactiveControls.registerEventListeners(vehicleType)
	SpecializationUtil.registerEventListener(vehicleType, "onLoad", interactiveControls);
	SpecializationUtil.registerEventListener(vehicleType, "onPostLoad", interactiveControls);
	SpecializationUtil.registerEventListener(vehicleType, "onUpdate", interactiveControls);
	SpecializationUtil.registerEventListener(vehicleType, "onLeave", interactiveControls);
	SpecializationUtil.registerEventListener(vehicleType, "onLeaveVehicle", interactiveControls);
	SpecializationUtil.registerEventListener(vehicleType, "onDraw", interactiveControls);
	SpecializationUtil.registerEventListener(vehicleType, "onRegisterActionEvents", interactiveControls);
end;

function interactiveControls:onLoad(savegame)
	self.spec_iC = {};
	self.spec_iC.iControls = {};
	self.spec_iC.interactiveControlsEvents = {};
	self.spec_iC.interactiveControlsEventIds = {};
	self.spec_iC.isICActive = false;
	local hasWipers, hasWiperSwitchAnimation = self:loadIControls();
	if hasWipers then 
		self.spec_iC.wipersMaxState = 0;
		for _, wiper in pairs(self.spec_wipers.wipers) do
			self.spec_iC.wipersMaxState = math.max(self.spec_iC.wipersMaxState, #wiper.states);
		end;
		if self.spec_iC.wipersMaxState > 0 then 
			self.spec_iC.wiperState = 0; 
			self.spec_iC.wiperSwitchAnimation = hasWiperSwitchAnimation;
		end;
	end;
	local soundFile = "data/sounds/vehicles/switch/switch_01.ogg"
	self.spec_iC.defaultSound = self:loadICSound(soundFile,1,1,getAppBasePath(),"defSnd");
end;

function interactiveControls:setLightTypeAndMsg(control, customMsg)
	local lightType = 1;
	local noTypeFound = false;
	local onMessage, offMessage = control.onMessage, control.offMessage;
	if control.lightType == "lowbeams" then
		lightType = 1; if not customMsg then onMessage = g_i18n:getText("input_TOGGLE_LIGHT_FRONT"); offMessage = onMessage; end;
	elseif control.lightType == "highbeams" then
		lightType = 2; if not customMsg then onMessage = g_i18n:getText("input_TOGGLE_HIGH_BEAM_LIGHT"); offMessage = onMessage; end;
	elseif control.lightType == "rearworklamps" then
		lightType = 3; if not customMsg then onMessage = g_i18n:getText("input_TOGGLE_WORK_LIGHT_BACK"); offMessage = onMessage; end;
	elseif control.lightType == "frontworklamps" then
		lightType = 4; if not customMsg then onMessage = g_i18n:getText("input_TOGGLE_WORK_LIGHT_FRONT"); offMessage = onMessage; end;
	elseif control.lightType == "hazardlamps" then
		lightType = 5; if not customMsg then onMessage = g_i18n:getText("input_TOGGLE_TURNLIGHT_HAZARD"); offMessage = onMessage; end;
	elseif control.lightType == "leftsignal" then
		lightType = 6; if not customMsg then onMessage = g_i18n:getText("input_TOGGLE_TURNLIGHT_LEFT"); offMessage = onMessage; end;
	elseif control.lightType == "rightsignal" then
		lightType = 7; if not customMsg then onMessage = g_i18n:getText("input_TOGGLE_TURNLIGHT_RIGHT"); offMessage = onMessage; end;
	elseif control.lightType == "beaconlamps" then
		lightType = 8; if not customMsg then onMessage = g_i18n:getText("input_TOGGLE_BEACON_LIGHTS"); offMessage = onMessage; end;
	elseif control.lightType == "allnonindicatorlights" then
		lightType = 9; if not customMsg then onMessage = g_i18n:getText("input_TOGGLE_LIGHTS"); offMessage = onMessage; end;
	else
		noTypeFound = true;
	end;

	return lightType, onMessage, offMessage, noTypeFound;
end;

function interactiveControls:loadIControls()
	local i,w,wA = 0,0,0;
	local btoN = {[false]=1,[true]=0};
	while true do
		local errorFound = false;
		local icString = string.format("vehicle.interactiveControls.interactiveControl(%d)",i);
		if not hasXMLProperty(self.xmlFile, icString) then break; end;
		local control = {};
		control.node = I3DUtil.indexToObject(self.components, getXMLString(self.xmlFile, icString.."#buttonNode"), self.i3dMappings);
		if control.node == nil then
			print("Error: Cannot find 'buttonNode' for interactive control "..i..". This control will not be loaded."); errorFound = true;
		else setVisibility(control.node,false); end;
		control.btnSize = MathUtil.clamp(Utils.getNoNil(getXMLFloat(self.xmlFile, icString..".options#btnSize"),0.1),0.01,1.0)/2;
		control.useDefaultSound = Utils.getNoNil(getXMLBool(self.xmlFile, icString..".sound_Options#useDefaultSound"),false);
		if not control.useDefaultSound then
			control.onSound = getXMLString(self.xmlFile, icString .. ".sound_Options#onSoundFile");
			control.onSndVolume = Utils.getNoNil(getXMLFloat(self.xmlFile, icString..".sound_Options#onSoundVolume"),1);
			control.loopOnSound = Utils.getNoNil(getXMLBool(self.xmlFile, icString..".sound_Options#loopOnSound"),false);
			control.loopOnSound = control.loopOnSound and control.onSound ~= nil; control.loopOnSound = btoN[control.loopOnSound];
			control.onSound = self:loadICSound(control.onSound,control.onSndVolume,control.loopOnSound,self.baseDirectory,"onSnd"..i);
			control.offSound = getXMLString(self.xmlFile, icString .. ".sound_Options#offSoundFile");
			control.offSndVolume = Utils.getNoNil(getXMLFloat(self.xmlFile, icString..".sound_Options#offSoundVolume"),1);
			control.loopOffSound = Utils.getNoNil(getXMLBool(self.xmlFile, icString..".sound_Options#loopOffSound"),false);
			control.loopOffSound = control.loopOffSound and control.offSound ~= nil; control.loopOffSound = btoN[control.loopOffSound];
			control.offSound = self:loadICSound(control.offSound,control.offSndVolume,control.loopOffSound,self.baseDirectory,"offSnd"..i);
		end;
		control.type = getXMLInt(self.xmlFile, icString .. "#type");
		if control.type == nil then
			print("Error: Cannot find 'type' for interactive control "..i..". This control will not be loaded."); errorFound = true;
		end;
		local customMsgs = false;
		control.onMessage = getXMLString(self.xmlFile, icString..".options#onMessage");
		control.offMessage =  getXMLString(self.xmlFile, icString..".options#offMessage");
		if control.onMessage ~= nil and control.offMessage ~= nil then
			if string.find(control.onMessage,"$l10n_") then control.onMessage = g_i18n:getText(control.onMessage); end;
			if string.find(control.offMessage,"$l10n_") then control.offMessage = g_i18n:getText(control.offMessage); end;
			customMsgs = true;
		end;
		control.animation = getXMLString(self.xmlFile, icString.."#animationName");
		control.animLastWasFwd = false;
		control.animSpeed = Utils.getNoNil(getXMLFloat(self.xmlFile, icString..".animation_Options#animSpeed"),1.0);
		local isDoor = Utils.getNoNil(getXMLBool(self.xmlFile, icString..".animation_Options#isDoor"),false);
		local hasAnimation = (control.animation ~= nil) and self:getAnimationExists(control.animation);
		if not customMsgs then
			if control.toggleOutDoorSound or control.type <= 1 then
				if isDoor then control.onMessage = g_i18n:getText("action_openDoor"); control.offMessage =  g_i18n:getText("action_closeDoor");
				else control.onMessage = g_i18n:getText("ic_openMsg"); control.offMessage =  g_i18n:getText("ic_closeMsg"); end;
			else
				control.onMessage = g_i18n:getText("ic_onMsg");
				control.offMessage =  g_i18n:getText("ic_offMsg");
			end;
		end;
		if control.type <= 1 then
			if not hasAnimation then
				print("Error: Cannot find 'animation' for interactive control "..i..". This control will not be loaded.");
				errorFound = true;
			else
				control.separateCtrls = Utils.getNoNil(getXMLBool(self.xmlFile, icString..".animation_Options#animUseSeparateControls"),false);
				control.animCanPause = Utils.getNoNil(getXMLBool(self.xmlFile, icString..".animation_Options#animCanPause"),false);
				control.toggleOutDoorSound = Utils.getNoNil(getXMLBool(self.xmlFile, icString..".animation_Options#toggleOutDoorSound"),false);
				if not customMsgs and control.separateCtrls then
					if control.toggleOutDoorSound or control.type <= 1 then
						control.onMessage = g_i18n:getText("input_interactiveControls_mouseLeft")..": "..g_i18n:getText("ic_openMsg").." | "..
						g_i18n:getText("input_interactiveControls_mouseRight")..": "..g_i18n:getText("ic_closeMsg");
						control.offMessage = control.onMessage;
					else
						control.onMessage = g_i18n:getText("input_interactiveControls_mouseLeft")..": "..g_i18n:getText("ic_onMsg").." | "..
						g_i18n:getText("input_interactiveControls_mouseRight")..": "..g_i18n:getText("ic_offMsg");
						control.offMessage = control.onMessage;
					end;
				end;
			end;
		elseif control.type == 2 then
			control.visibilityNode = I3DUtil.indexToObject(self.components, getXMLString(self.xmlFile, icString.."#visibilityNode"), self.i3dMappings);
			if control.visibilityNode == nil then
				print("Error: Cannot find 'visibilityNode' for interactive control "..i..". This control will not be loaded."); errorFound = true;
			else setVisibility(control.visibilityNode,false); end;
			control.isLight = Utils.getNoNil(getXMLBool(self.xmlFile, icString..".visibility_Options#isLight"), false);
			if not customMsgs and isLight then
				control.onMessage = g_i18n:getText("action_turnOnLight");
				control.offMessage =  g_i18n:getText("action_turnOffLight");
			end;
		elseif control.type == 3 then
			control.lightType = getXMLString(self.xmlFile, icString .. "#lightType");
			local error = false;
			if control.lightType == nil then
				errorFound = true;
			else
				control.lightType = string.lower(control.lightType);
				control.lightType, control.onMessage, control.offMessage, errorFound = self:setLightTypeAndMsg(control, customMsgs);
			end;
			if errorFound then print("Error: Cannot find 'lightType' for interactive control "..i..". This control will not be loaded."); errorFound = true; end;
		elseif control.type == 4 and not customMsgs then
			control.onMessage = g_i18n:getText("action_startMotor");
			control.offMessage =  g_i18n:getText("action_stopMotor");
		elseif control.type == 5 then
			w = w+1;
			if not customMsgs then
				control.onMessage = g_i18n:getText("ic_onMsg");
				control.offMessage =  g_i18n:getText("ic_offMsg");
			end;
			if hasAnimation then wA = wA+1; end;
		end;
		if control.type > 1 and hasAnimation then
			control.separateCtrls = false;
			control.animCanPause = false;
		end;
		control.useOnMsg = false;
		if not errorFound then table.insert(self.spec_iC.iControls, control); end;
		i = i + 1;
	end;
	if interactiveControls.debugging then print(#self.spec_iC.iControls.." Interactive Controls Loaded to "..FSBaseMission:getVehicleName(self)); end;
	return w>0,wA>0;
end;

function interactiveControls:onPostLoad(savegame)
	local spec = self.spec_iC;
	if savegame ~= nil and #spec.iControls > 0 then
		local ia,iv=0,0;
		for _, control in pairs(spec.iControls) do
			if control.type <=2 then
				if control.animation ~= nil then
					local animName = getXMLString(savegame.xmlFile, string.format(savegame.key..".interactiveControls.animation(%d)", ia).."#name");
					if animName ~= nil and animName == control.animation then
						local animTime = Utils.getNoNil(getXMLFloat(savegame.xmlFile, string.format(savegame.key..".interactiveControls.animation(%d)", ia).."#time"), 0);
						self:setAnimationTime(control.animation, animTime, true);
						if control.animLastWasFwd ~= nil then
							control.animLastWasFwd = Utils.getNoNil(getXMLBool(savegame.xmlFile, string.format(savegame.key..".interactiveControls.animation(%d)", ia).."#lastDir"), false);
						end;
						ia=ia+1;
					end;
				end;
				if control.visibilityNode ~= nil then
					local visNodeName = getXMLString(savegame.xmlFile, string.format(savegame.key..".interactiveControls.visibility(%d)", iv).."#name");
					if visNodeName ~= nil and visNodeName == getName(control.visibilityNode) then
						local visible = Utils.getNoNil(getXMLBool(savegame.xmlFile, string.format(savegame.key..".interactiveControls.visibility(%d)", iv).."#isVisible"), false);
						setVisibility(control.visibilityNode,visible);
						iv=iv+1;
					end;
				end;
			end;
		end;
	end;
end;

function interactiveControls:saveToXMLFile(xmlFile, key)
	local spec = self.spec_iC;
	if #spec.iControls > 0 then
		key = string.gsub(key,self.customEnvironment..".","")
		local ia,iv=0,0;
		for _, control in pairs(spec.iControls) do
			if control.type <=2 then
				if control.animation ~= nil then
					setXMLString(xmlFile, string.format(key..".animation(%d)", ia).."#name", control.animation);
					setXMLFloat(xmlFile, string.format(key..".animation(%d)", ia).."#time", self:getAnimationTime(control.animation));
					if control.animLastWasFwd ~= nil then
						setXMLBool(xmlFile, string.format(key..".animation(%d)", ia).."#lastDir", control.animLastWasFwd);
					end;
					ia=ia+1
				end;
				if control.visibilityNode ~= nil then
					setXMLString(xmlFile, string.format(key..".visibility(%d)", iv).."#name", getName(control.visibilityNode));
					setXMLBool(xmlFile, string.format(key..".visibility(%d)", iv).."#isVisible", getVisibility(control.visibilityNode));
					iv=iv+1;
				end;
			end;
		end;
	end;
end;

function interactiveControls:onRegisterActionEvents(isActiveForInput, isActiveForInputIgnoreSelection)
	if self.isClient then
		self:clearActionEventsTable(self.interactiveControlsEvents);
		self.spec_iC.interactiveControlsEventIds = {};
		if isActiveForInputIgnoreSelection then
			local _, actionEventId = self:addActionEvent(self.spec_iC.interactiveControlsEvents, 'interactiveControls_toggleKey', self, interactiveControls.iCToggleKeyEvent, false, true, false, true, nil);
			g_inputBinding:setActionEventTextVisibility(actionEventId, true);
			g_inputBinding:setActionEventTextPriority(actionEventId, GS_PRIO_HIGH);
			table.insert(self.spec_iC.interactiveControlsEventIds, actionEventId);
			--
			_, actionEventId = self:addActionEvent(self.spec_iC.interactiveControlsEvents, 'interactiveControls_mouseLeft', self, interactiveControls.mouseICEvent, false, true, false, true, nil);
			g_inputBinding:setActionEventTextVisibility(actionEventId, false);
			g_inputBinding:setActionEventTextPriority(actionEventId, GS_PRIO_VERYLOW);
			table.insert(self.spec_iC.interactiveControlsEventIds, actionEventId);
			--
			_, actionEventId = self:addActionEvent(self.spec_iC.interactiveControlsEvents, 'interactiveControls_mouseRight', self, interactiveControls.mouseICEvent, false, true, false, true, nil);
			g_inputBinding:setActionEventTextVisibility(actionEventId, false);
			g_inputBinding:setActionEventTextPriority(actionEventId, GS_PRIO_VERYLOW);
			table.insert(self.spec_iC.interactiveControlsEventIds, actionEventId);
		end;
	end;
end;

function interactiveControls:onUpdate(dt)
	if self.isClient then
		if self:getIsActiveForInput(true,true) then
			local spec = self.spec_iC;
			if #spec.iControls > 0 then
				local ctrlFound = 0;
				for _, control in pairs(spec.iControls) do
					if spec.isICActive then
						if not getVisibility(control.node) then
							setVisibility(control.node, true);
						end;
						local c = self:updateIControls(control, dt)
						ctrlFound = ctrlFound+c;
					else
						spec.target = nil;
						if getVisibility(control.node) then
							setVisibility(control.node, false);
							setScale(control.node,1,1,1);
						end;
					end;
					if ctrlFound <= 0 then self.spec_iC.target = nil; end;
					self:updateICtrlSounds(control);
				end;
				self:updateIndoorSounds();
				self:checkICCamera(spec.isICActive);
				if spec.wiperState ~= nil then self:updateICWipers(); end;
			end;
		end;
	end;
end;

function interactiveControls:updateIControls(control, dt)
	local x,y,_ = project(unpack({getWorldTranslation(control.node)}));
	local tolerance = 0.1;
	local icTargetXFound = (g_lastMousePosX > (x-control.btnSize/2) and (g_lastMousePosX < (x+control.btnSize/2)));
	local icTargetYFound = (g_lastMousePosY > (y-control.btnSize/2) and (g_lastMousePosY < (y+control.btnSize/2)));
	if icTargetXFound and icTargetYFound then
		local scale,_,_ = getScale(control.node);
		scale = math.min(scale + (0.1 * (dt/100)),1.25);
		setScale(control.node,scale,scale,scale);
		self.spec_iC.target = control;
		self:updateICtrlMsgs(control);
		return 1;
	else
		local scale,_,_ = getScale(control.node);
		if scale > 1.0 then
			local scale = math.max(scale - (0.1 * (dt/100)),1);
			setScale(control.node,scale,scale,scale);
		elseif scale ~= 1.0 then
			setScale(control.node,1,1,1);
		end;
		return 0;
	end;
end;

function interactiveControls:checkICCamera(isActive)
	if isActive then
		if not self:getIsIndoorCamera() then
			self:killIC();
		end;
	end;
end;

function interactiveControls:onLeave()
	self:killIC();
end;

function interactiveControls:onLeaveVehicle()
	self:killIC();
end;

function interactiveControls:killIC()
	for _,camera in pairs(self.spec_enterable.cameras) do
		if camera.isInside then
			camera.isRotatable = true; break;
		end;
	end;
	self.spec_iC.target = nil;
	self.spec_iC.isICActive =  false;
	g_inputBinding:setShowMouseCursor(false);
end;

function interactiveControls:getIsIndoorCamera()
	local indoorCam = false;
	local cam = self:getActiveCamera();
	if cam ~= nil then
		for _,camera in pairs(self.spec_enterable.cameras) do
			if camera == cam and camera.isInside then
				indoorCam = true; break;
			end;
		end;
	end;
	return indoorCam;
end;

function interactiveControls:updateICtrlMsgs(control)
	if control.onMessage ~= control.offMessage then
		local toggled = false;
		if control.type == 1 then-- animation
			if not control.separateCtrls then
				if not self:getIsAnimationPlaying(control.animation) then
					if control.animCanPause then
						toggled = control.animLastWasFwd;
					else
						toggled = self:getAnimationTime(control.animation) > 0;
					end;
				end;
			end;
		elseif control.type == 2 then-- visibility
			toggled = getVisibility(control.visibilityNode);
		elseif control.type == 3 then-- lights
			toggled = toggled;
			if control.lightType == 1 then
				toggled = bitAND(self:getLightsTypesMask(), 1) ~= 0;
			elseif control.lightType == 2 then
				toggled = bitAND(self:getLightsTypesMask(), 8) ~= 0;
			elseif control.lightType == 3 then
				toggled = bitAND(self:getLightsTypesMask(), 2) ~= 0;
			elseif control.lightType == 4 then
				toggled = bitAND(self:getLightsTypesMask(), 4) ~= 0;
			elseif control.lightType == 5 then
				toggled = self:getTurnLightState() == Lights.TURNLIGHT_HAZARD;
			elseif control.lightType == 6 then
				toggled = self:getTurnLightState() == Lights.TURNLIGHT_LEFT;
			elseif control.lightType == 7 then
				toggled = self:getTurnLightState() == Lights.TURNLIGHT_RIGHT;
			elseif control.lightType == 8 then
				toggled = self:getBeaconLightsVisibility();
			elseif control.lightType == 9 then
				toggled = self:getLightsTypesMask() ~= 0;
			end;
		elseif control.type == 4 then-- start/stop vehicle
			toggled = self:getIsMotorStarted();
		elseif control.type == 5 then
			toggled = self.spec_iC.wiperState == self.spec_iC.wipersMaxState;
		end;
		if control.useOnMsg ~= toggled then
			control.useOnMsg = toggled;
		end;
	end;
end;

function interactiveControls:updateICtrlSounds(control)
	if control.animation ~= nil then
		if not self:getIsAnimationPlaying(control.animation) then
			if control.loopOnSound then
				if g_soundManager:getIsSamplePlaying(control.onSound) then
					g_soundManager:stopSample(control.onSound);
				end;
			end;
			if control.loopOffSound then
				if g_soundManager:getIsSamplePlaying(control.offSound) then
					g_soundManager:stopSample(control.offSound);
				end;
			end;
		end;
	end;
end;

function interactiveControls:updateIndoorSounds()
	if self:getIsEntered() then
		if self:getIsIndoorCamera() then
			local i = 0;
			for _, control in pairs(self.spec_iC.iControls) do
				if control.animation ~= nil and control.toggleOutDoorSound ~= nil then
					if control.toggleOutDoorSound then
						if self:getAnimationTime(control.animation) > 0 then
							i = i+1;
						end;
					end;
				end;
			end;
			g_soundManager:setIsIndoor(i==0);
		else
			g_soundManager:setIsIndoor(false);
		end;
	end;
end;

function interactiveControls:iCToggleKeyEvent(actionName, inputValue, callbackState, isAnalog)
	if inputValue >= 1 then
		if actionName == "interactiveControls_toggleKey" then
			local spec = self.spec_iC;
			if not spec.isICActive and self:getIsIndoorCamera() then
				spec.isICActive = true;
				self:getActiveCamera().isRotatable = false;
				g_inputBinding:setShowMouseCursor(true);
			else
				spec.isICActive = false;
				self:getActiveCamera().isRotatable = true;
				g_inputBinding:setShowMouseCursor(false);
			end;
		end;
	end;
end;

function interactiveControls:mouseICEvent(actionName, inputValue, callbackState, isAnalog)
	if inputValue >= 1 then
		local spec = self.spec_iC;
		if spec.isICActive and self:getIsIndoorCamera() then
			if spec.target ~= nil then
				local targetIndex = 0;-----------------------------------
				for i,c in pairs(spec.iControls) do
					if spec.target == c then targetIndex = i; break; end;
				end;-----------------------------------------------------
				if actionName == "interactiveControls_mouseLeft" then
					self:useIControlEvent(targetIndex, 1);--spec.target
				elseif actionName == "interactiveControls_mouseRight" then
					self:useIControlEvent(targetIndex, 2);--spec.target
				end;
			end;
		end;
	end;
end;

function interactiveControls:useIControlEvent(targetIndex, btn, noEventSend)
	interactiveControlsEvent.sendEvent(self, targetIndex, btn, noEventSend);
	local control = self.spec_iC.iControls[targetIndex];------------------
	self:useIControl(control, btn);
end;

function interactiveControls:useIControl(control, btn)
	local function runAnimation(control, fwd)
		local forced = (fwd~=nil);
		if btn < 2 then --left click
			if not forced then
				if not self:getIsAnimationPlaying(control.animation) then
					local playAnim = not control.animLastWasFwd or control.separateCtrls;
					if playAnim then
						if self:getAnimationTime(control.animation) < 1 then
							local speed = control.animSpeed;
							local animation = self.spec_animatedVehicle.animations[control.animation];
							if animation.currentTime > 0 then speed = speed*((animation.duration+animation.currentTime)/animation.duration); end;
							self:playICSound(control, true);
							self:playAnimation(control.animation, speed);
							control.animLastWasFwd = true;
						end;
					elseif not control.separateCtrls then
						if self:getAnimationTime(control.animation) > 0 then
							local speed = control.animSpeed;
							local animation = self.spec_animatedVehicle.animations[control.animation];
							if animation.currentTime < animation.duration then speed = speed*((animation.duration+(animation.duration-animation.currentTime))/animation.duration); end;
							self:playICSound(control, false);
							self:playAnimation(control.animation, -speed);
							control.animLastWasFwd = false;
						end;
					end;
				elseif control.animCanPause then
					if self:getIsAnimationPlaying(control.animation) then
						self:stopAnimation(control.animation);
						control.animLastWasFwd = not control.animLastWasFwd;
						if control.onSound ~= nil then g_soundManager:stopSample(control.onSound); end;
						if control.offSound ~= nil then g_soundManager:stopSample(control.offSound); end;
					end;
				end;
			else
				if fwd then
					self:playAnimation(control.animation, control.animSpeed);
				else
					self:playAnimation(control.animation, -control.animSpeed);
				end;
			end;
		elseif not forced then --right click
			if control.separateCtrls then
				if not self:getIsAnimationPlaying(control.animation) then
					if self:getAnimationTime(control.animation) > 0 then
						local speed = control.animSpeed;
						local animation = self.spec_animatedVehicle.animations[control.animation];
						if animation.currentTime < animation.duration then speed = speed*((animation.duration+(animation.duration-animation.currentTime))/animation.duration); end;
						self:playICSound(control, false);
						self:playAnimation(control.animation, -speed);
						control.animLastWasFwd = false;
					end;
				elseif control.animCanPause then
					if self:getIsAnimationPlaying(control.animation) then
						self:stopAnimation(control.animation);
						if control.onSound ~= nil then g_soundManager:stopSample(control.onSound); end;
						if control.offSound ~= nil then g_soundManager:stopSample(control.offSound); end;
					end;
				end;
			end;
		end;
	end;
	if control.type == 1 then-- animation
		runAnimation(control);
	elseif control.type == 2 then-- visibility
		if btn < 2 then --left click
			local visibility = not getVisibility(control.visibilityNode);
			setVisibility(control.visibilityNode, visibility);
			if control.animation ~= nil then runAnimation(control, visibility); end;
			if control.isLight then
				if visibility then
					if control.offSound ~= nil then g_soundManager:stopSample(control.offSound); end;
					self:playICSound(control, true);
				else
					if control.onSound ~= nil then g_soundManager:stopSample(control.onSound); end;
					self:playICSound(control, false);
				end;
			end;
		end;
	elseif control.type == 3 then-- lights
		if btn < 2 then --left click
			local toggled = false;
			local spec = self.spec_lights;
			if control.lightType == 1 then--lowbeams
				local lightsTypesMask = bitXOR(spec.lightsTypesMask, 1);
				self:setLightsTypesMask(lightsTypesMask, true, false);
				toggled = bitAND(self:getLightsTypesMask(), 1) ~= 0;
				self:playICSound(control, toggled);
			elseif control.lightType == 2 then--higbeams
				local lightsTypesMask = bitXOR(spec.lightsTypesMask, 8);
				self:setLightsTypesMask(lightsTypesMask, true, false);
				toggled = bitAND(self:getLightsTypesMask(), 8) ~= 0;
				self:playICSound(control, toggled);
			elseif control.lightType == 3 then--rearWorkLamps
				local lightsTypesMask = bitXOR(spec.lightsTypesMask, 2);
				self:setLightsTypesMask(lightsTypesMask, true, false);
				toggled = bitAND(self:getLightsTypesMask(), 2) ~= 0;
				self:playICSound(control, toggled);
			elseif control.lightType == 4 then--frontWorkLamps
				local lightsTypesMask = bitXOR(spec.lightsTypesMask, 4);
				self:setLightsTypesMask(lightsTypesMask, true, false);
				toggled = bitAND(self:getLightsTypesMask(), 4) ~= 0;
				self:playICSound(control, toggled);
			elseif control.lightType == 5 then--hazardLamps
				toggled = self:getTurnLightState() == Lights.TURNLIGHT_HAZARD;
				if not toggled then self:setTurnLightState(Lights.TURNLIGHT_HAZARD, true, false);
				else self:setTurnLightState(Lights.TURNLIGHT_OFF, true, false); end;
				self:playICSound(control, toggled);
			elseif control.lightType == 6 then--leftSignal
				toggled = self:getTurnLightState() == Lights.TURNLIGHT_LEFT;
				if not toggled then self:setTurnLightState(Lights.TURNLIGHT_LEFT, true, false);
				else self:setTurnLightState(Lights.TURNLIGHT_OFF, true, false); end;
				self:playICSound(control, toggled);
			elseif control.lightType == 7 then--rightSignal
				toggled = self:getTurnLightState() == Lights.TURNLIGHT_RIGHT;
				if not toggled then self:setTurnLightState(Lights.TURNLIGHT_RIGHT, true, false);
				else self:setTurnLightState(Lights.TURNLIGHT_OFF, true, false); end;
				self:playICSound(control, toggled);
			elseif control.lightType == 8 then--rightSignal
				toggled = not self:getBeaconLightsVisibility();
				self:setBeaconLightsVisibility(toggled, true, false);
				self:playICSound(control, toggled);
			elseif control.lightType == 9 then--allnonindicatorlights
				spec:setNextLightsState();
				toggled = self:getLightsTypesMask() ~= 0;
				self:playICSound(control, toggled);
			end;
			if control.animation ~= nil then runAnimation(control, toggled); end;
		end;
	elseif control.type == 4 then-- start/stop vehicle
		if btn < 2 then --left click
			local engineRunning = self:getIsMotorStarted();
			if engineRunning then
				self:stopMotor();
				if control.animation ~= nil then runAnimation(control, false); end;
				self:playICSound(control, false);
			else
				self:startMotor();
				if control.animation ~= nil then runAnimation(control, true); end;
				self:playICSound(control, true);
			end;
		end;
	elseif control.type == 5 then-- wipers
		if btn < 2 then --left click
			if self.spec_iC.wiperOverRide == nil then
				self.spec_iC.wiperState = self.spec_iC.wiperState + 1;
				if self.spec_iC.wiperState > self.spec_iC.wipersMaxState then self.spec_iC.wiperState = 0; end;
				local toggled = self.spec_iC.wiperState == self.spec_iC.wipersMaxState;
				self:playICSound(control, toggled);
				if control.animation ~= nil then
					if self.spec_iC.wiperState == 1 then
						runAnimation(control, true);
					elseif self.spec_iC.wiperState == 0 then
						runAnimation(control, false);
					end;
				end;
			end;
		end;
	end;
end;

function interactiveControls:playICSound(control,toggled)
	if self.isClient then
		if control.onSound ~= nil and toggled then
			g_soundManager:playSample(control.onSound);
		elseif control.useDefaultSound then
			g_soundManager:playSample(self.spec_iC.defaultSound);
		end;
		if control.offSound ~= nil and not toggled then
			g_soundManager:playSample(control.offSound);
		end;
	end;
end;

function interactiveControls:updateICWipers()
	local spec = self.spec_wipers;
	if spec.hasWipers and self:getIsControlled() then
		spec.lastRainScale = g_currentMission.environment.weather:getRainFallScale();
		for _, wiper in pairs(spec.wipers) do
			local stateIdToUse = 0;
			if Wipers.forcedState == 0 and spec.lastRainScale <= 0 then
				 stateIdToUse = MathUtil.clamp(self.spec_iC.wiperState, 0, #wiper.states);
				self.spec_iC.wiperOverRide = nil;
				if self.spec_iC.wiperSwitchAnimation then
					for _, control in pairs(self.spec_iC.iControls) do
						if control.type == 5 then
							if control.animation ~= nil then 
								if spec.usedICwiperAnimation ~= nil and self:getAnimationTime(control.animation) >= 1 then
									if control.animation ~= nil then self:playAnimation(control.animation, -control.animSpeed); end;
									spec.usedICwiperAnimation = nil;
								end;
							end;
						end;
					end;
				end;
			else
				if self.spec_iC.wiperSwitchAnimation then
					for _, control in pairs(self.spec_iC.iControls) do
						if control.type == 5 then
							if control.animation ~= nil then 
								if self:getAnimationTime(control.animation) < 1 then
									if control.animation ~= nil then self:playAnimation(control.animation, control.animSpeed); end;
									spec.usedICwiperAnimation = true;
								end;
							end;
						end;
					end;
				end;
				self.spec_iC.wiperState = 0;
				self.spec_iC.wiperOverRide = true;
				return;
			end;
			if stateIdToUse > 0 then
				local currentState = wiper.states[stateIdToUse];
				if self:getAnimationTime(wiper.animName) == 1 then
					self:playAnimation(wiper.animName, -currentState.animSpeed, 1, true);
				end;
				if (wiper.nextStartTime == nil or wiper.nextStartTime < g_currentMission.time) and not self:getIsAnimationPlaying(wiper.animName) then
					self:playAnimation(wiper.animName, currentState.animSpeed, 0, true);
					wiper.nextStartTime = nil;
				end;
				if wiper.nextStartTime == nil then
					wiper.nextStartTime = g_currentMission.time + wiper.animDuration / currentState.animSpeed + currentState.animPause;
				end;
			elseif self:getAnimationTime(wiper.animName) > 0 then
				self:playAnimation(wiper.animName, -1);
			end;
		end;
	end;
end;

function interactiveControls:onDraw(dt)
	local spec = self.spec_iC;
	if spec.target ~= nil then
		local icMsg = "";
		if spec.target.useOnMsg then
			icMsg = spec.target.offMessage;
		else
			icMsg = spec.target.onMessage;
		end;
		local uiScale = g_gameSettings:getValue("uiScale");
		local textSize = 0.05*uiScale;
		local oX = textSize * HUDTextDisplay.SHADOW_OFFSET_FACTOR;
		setTextBold(false);
		setTextColor(0,0,0,1);
		setTextAlignment(RenderText.ALIGN_CENTER);
		renderText(0.5+oX,0.06-oX,textSize,icMsg);
		setTextColor(0,0.75,1,1);
		renderText(0.5,0.06,textSize,icMsg);
	end;
end;

function interactiveControls:loadICSound(sound, volume, loop, baseDir, prefix)
	if sound == nil then return; end;
	local sourceToCopy = self.spec_lights.samples.turnLight;
	local fileFound = fileExists(baseDir..sound);
	if sourceToCopy ~= nil and fileFound then
		local sample = ListUtil.copyTable(self.spec_lights.samples.turnLight);
		sample.modifiers = ListUtil.copyTable(self.spec_lights.samples.turnLight.modifiers);
		sample.filename = Utils.getFilename(sound, baseDir);
		local soundFilename = StringUtil.getFilenameFromPath(sound);
		sample.sampleName = prefix..string.sub(soundFilename,1, string.len(soundFilename)-4);
		sample.indoorAttributes.volume = volume;
		sample.indoorAttributes.pitch = 1;
		sample.indoorAttributes.lowpassGain = 1;
		sample.outdoorAttributes.volume = volume;
		sample.outdoorAttributes.pitch = 1;
		sample.outdoorAttributes.lowpassGain = 1;
		sample.loops = loop;
		sample.soundNode = createAudioSource(sample.sampleName, sample.filename, 3,2, sample.current.volume, loop);--3=outerRadius, 2=innerRadius
		sample.soundSample = getAudioSourceSample(sample.soundNode);
		setAudioSourceAutoPlay(sample.soundNode, false);
		link(self.rootNode, sample.soundNode);
		sample.linkNode = self.rootNode;
		setTranslation(sample.soundNode, 0, 1, 0);
		setSampleGroup(sample.soundSample, AudioGroup.VEHICLE);
		sample.audioGroup = AudioGroup.VEHICLE;
		sample.modifierTargetObject = self;
		g_soundManager.samples[sample] = sample;
		table.insert(g_soundManager.orderedSamples, sample);
		if interactiveControls.debugging then print("Loaded IC Sound file: "..soundFilename); end;
		return sample;
	else
		if sourceToCopy == nil then
			print("Error: Vehicle 'turnLight' sound required for 'interactive control' sounds to work.");
		elseif not fileFound then
			print("Error: Cannot find '"..baseDir..sound.."' for interactive control.");
		end;
		return;
	end;
end;

interactiveControlsEvent = {}; --multiplayer event
interactiveControlsEvent_mt = Class(interactiveControlsEvent, Event);
InitEventClass(interactiveControlsEvent,"interactiveControlsEvent");

function interactiveControlsEvent:emptyNew()
    local self = Event:new(interactiveControlsEvent_mt);
    self.className = "interactiveControlsEvent";
    return self;
end;
function interactiveControlsEvent:new(vehicle,control, btn)
	local self = interactiveControlsEvent:emptyNew();
    self.vehicle = vehicle;
    self.control = control;
    self.btn = btn;
    return self;
end;
function interactiveControlsEvent:readStream(streamId,connection)
    self.vehicle = NetworkUtil.readNodeObject(streamId);
    self.control = streamReadInt8(streamId);
    self.btn = streamReadInt8(streamId);
    self:run(connection);
end;
function interactiveControlsEvent:writeStream(streamId,connection)
    NetworkUtil.writeNodeObject(streamId,self.vehicle);
    streamWriteInt8(streamId,self.control);
    streamWriteInt8(streamId,self.btn);
end;
function interactiveControlsEvent:run(connection)
    self.vehicle:useIControlEvent(self.control,self.btn, true);
    if not connection:getIsServer() then
        g_server:broadcastEvent(interactiveControlsEvent:new(self.vehicle,self.control,self.btn),nil,connection,self.vehicle);
    end;
end;
function interactiveControlsEvent.sendEvent(vehicle,control,btn,noEventSend)
    if noEventSend == nil or noEventSend == false then
        if g_server ~= nil then
            g_server:broadcastEvent(interactiveControlsEvent:new(vehicle,control,btn),nil,nil,vehicle);
        else
            g_client:getServerConnection():sendEvent(interactiveControlsEvent:new(vehicle,control,btn));
        end;
    end;
end;