----------------------------------------------------------------------------
-- @Author: ViperGTS96-----------------------------------------------------
----------------------------------------------------------------------------
--------------------"The simplest design is the best design." --------------
----------------------------------------------------------------------------
----------------------------------------------------------------------------

wetVehicles = {};
wetVehicles.runUpdate = true;
wetVehicles.buttonDown = false;
wetVehicles.rainInit = false;
wetVehicles.drySpeed = 0.00001;
wetVehicles.wetSpeed = 0.0001;

function wetVehicles:loadMap()
	g_currentMission.wetVehicles = {};
	g_currentMission.wetVehicles.isRaining = false;
end;

function wetVehicles:getIsVehicleInBuilding()
	--if g_currentMission.isInsideBuilding then return true; end;
	local vehicleHeight = Utils.getNoNil(self.sizeWidth,4.5);
	local x,y,z = getWorldTranslation(self.rootNode);
	raycastAll(x,y+vehicleHeight,z, 0,1,0, "vehicleInBuildingRaycastCallback", 30.0, self, 255);
	return self.isWithinBuilding;
end;

function wetVehicles:vehicleInBuildingRaycastCallback(hitObjectId, x,y,z, distance)
	if hitObjectId ~= nil then
		local objectType = getRigidBodyType(hitObjectId);
		if objectType ~= "Dynamic" and objectType ~= "Kinematic" then
			self.isWithinBuilding = true;
		end;
	end;
end

function wetVehicles:getIsRaining()
	if Wipers.forcedState > 0 then
		return true;
	else
		local rainScale = g_currentMission.environment.weather:getRainFallScale()
		local timeSinceLastRain = g_currentMission.environment.weather:getTimeSinceLastRain()
		if not (rainScale <= 0.1 and timeSinceLastRain > 20) then
			return true;
		end;
	end;
	return false;
end;

function wetVehicles:getIsTireInWater(vehicle,node)
	if vehicle.spec_wheels ~= nil then
		for _,wheel in pairs(vehicle.spec_wheels.wheels) do
			if wheel.wetTireDetectNode ~= nil then
				local wetTireY = g_currentMission.waterY;
				if g_currentMission.waterBodyFX ~= nil then
					wetTireY = g_currentMission.waterBodyFX.waterY;
				end;
				local _,y,_ = getWorldTranslation(wheel.wetTireDetectNode);
				if y <= wetTireY then
					return true;
				end;
			end;
		end;
	end;
	return false;
end;

function wetVehicles:update(dt)
	if wetVehicles.runUpdate then
		local isRaining = wetVehicles:getIsRaining();
		g_currentMission.wetVehicles.isRaining = isRaining; -- for use in other rain scripts.
		if isRaining and not wetVehicles.rainInit then wetVehicles.rainInit = true; end;
		if wetVehicles.rainInit then
			for _,vehicle in pairs(g_currentMission.vehicles) do
				if vehicle.spec_wetVehicles ~= nil then
					if vehicle.getIsVehicleInBuilding == nil then
						vehicle.getIsVehicleInBuilding = wetVehicles.getIsVehicleInBuilding;
						vehicle.vehicleInBuildingRaycastCallback = wetVehicles.vehicleInBuildingRaycastCallback;
					end;
					local vehicleInRain = isRaining;
					if isRaining then
						vehicle.isWithinBuilding = false;
						vehicleInRain = not vehicle:getIsVehicleInBuilding();
					end;
					if vehicle.spec_washable ~= nil and vehicle.spec_washable.washableNodes ~= nil then
						for _, nodeData in ipairs(vehicle.spec_washable.washableNodes) do
							if nodeData.nodeMinShine ~= nil then
								for _, node in pairs(nodeData.nodes) do
									if not wetVehicles:getIsTireInWater(vehicle,node) then
										local shine,metalness,_,_ = getShaderParameter(node, "SM");
										if vehicleInRain then
											if shine < 1 then
												setShaderParameter(node, "SM", shine+(wetVehicles.wetSpeed*dt), metalness, 0, 0, false);
											end;
										else
											if shine > nodeData.nodeMinShine then
												setShaderParameter(node, "SM", shine-(wetVehicles.drySpeed*dt), metalness, 0, 0, false);
											elseif shine ~= nodeData.nodeMinShine then
												setShaderParameter(node, "SM", nodeData.nodeMinShine, metalness, 0, 0, false);
											end;
										end;
									end;
								end;
							end;
						end;
					end;
				end;
			end;
		end;
	elseif g_currentMission.wetVehicles.isRaining then
		g_currentMission.wetVehicles.isRaining = false;
	end;
	
	if wetVehicles.rainInit then
		if Input.isKeyPressed(Input.KEY_ralt) then
			if not wetVehicles.buttonDown and Input.isKeyPressed(Input.KEY_backspace) then
				wetVehicles.buttonDown = true;
				wetVehicles.runUpdate = not wetVehicles.runUpdate;
			end;
		elseif wetVehicles.buttonDown then
			wetVehicles.buttonDown = false;
		end;
	end;
end;

--[[function wetVehicles:postLoadMap() -- basic function to load all vehicle's nodes. (obsolete)
	for _,vehicle in pairs(g_currentMission.vehicles) do
		if vehicle.spec_washable ~= nil and vehicle.spec_washable.washableNodes ~= nil then
			for _, nodeData in ipairs(vehicle.spec_washable.washableNodes) do
				for _, node in pairs(nodeData.nodes) do
					if getHasShaderParameter(node, "SM") then
						local shine,_,_,_ = getShaderParameter(node, "SM");
						nodeData.nodeMinShine = shine;
					end;
				end;
			end;
		end;
	end;
end;
Mission00.onStartMission = Utils.appendedFunction(Mission00.onStartMission, wetVehicles.postLoadMap);]]

function wetVehicles:wipersUpdateTick(superfunc, dt, isActiveForInput, isActiveForInputIgnoreSelection, isSelected)
	local spec = self.spec_wipers;
	if spec.hasWipers and self:getIsControlled() then
		spec.lastRainScale = g_currentMission.environment.weather:getRainFallScale();
		for _, wiper in pairs(spec.wipers) do
			local stateIdToUse = 0;
			if self:getIsActiveForWipers() and spec.lastRainScale > 0 then
				for stateIndex, state in ipairs(wiper.states) do
					if spec.lastRainScale <= state.maxRainValue then
						stateIdToUse = stateIndex;
						break;
					end;
				end;
			end;
			if Wipers.forcedState ~= 0 then
				stateIdToUse = MathUtil.clamp(Wipers.forcedState, 1, #wiper.states);
			end;
			--
			if self.isWithinBuilding ~= nil and self.isWithinBuilding then
				stateIdToUse = 0;
				if self:getAnimationTime(wiper.animName) == 1 then
					self:playAnimation(wiper.animName, -(wiper.states[1].animSpeed), 1, true);
				end;
			end;
			--
			if stateIdToUse > 0 then
				local currentState = wiper.states[stateIdToUse];
				if self:getAnimationTime(wiper.animName) == 1 then
					self:playAnimation(wiper.animName, -currentState.animSpeed, 1, true);
				end;
				if (wiper.nextStartTime == nil or wiper.nextStartTime < g_currentMission.time) and not self:getIsAnimationPlaying(wiper.animName) then
					self:playAnimation(wiper.animName, currentState.animSpeed, 0, true);
					wiper.nextStartTime = nil;
				end;
				if wiper.nextStartTime == nil then
					wiper.nextStartTime = g_currentMission.time + wiper.animDuration / currentState.animSpeed + currentState.animPause;
				end;
			end;
		end;
	end;
end;

Wipers.onUpdateTick = Utils.overwrittenFunction(Wipers.onUpdateTick,wetVehicles.wipersUpdateTick);
addModEventListener(wetVehicles);